<?php

/**
 * This file is part of the authbucket/oauth2 package.
 *
 * (c) Wong Hoi Sing Edison <hswong3i@pantarei-design.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace AuthBucket\OAuth2\Security\Firewall;

use AuthBucket\OAuth2\Exception\InvalidRequestException;
use AuthBucket\OAuth2\Security\Authentication\Token\AccessTokenToken;
use AuthBucket\OAuth2\TokenType\TokenTypeHandlerFactoryInterface;
use AuthBucket\OAuth2\Validator\Constraints\AccessToken;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\Security\Core\Authentication\AuthenticationManagerInterface;
use Symfony\Component\Security\Core\SecurityContextInterface;
use Symfony\Component\Security\Http\Firewall\ListenerInterface;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\ValidatorInterface;

/**
 * ResourceListener implements OAuth2 resource endpoint authentication.
 *
 * @author Wong Hoi Sing Edison <hswong3i@pantarei-design.com>
 */
class ResourceListener implements ListenerInterface
{
    protected $providerKey;
    protected $securityContext;
    protected $authenticationManager;
    protected $validator;
    protected $tokenTypeHandlerFactory;

    public function __construct(
        $providerKey,
        SecurityContextInterface $securityContext,
        AuthenticationManagerInterface $authenticationManager,
        ValidatorInterface $validator,
        TokenTypeHandlerFactoryInterface $tokenTypeHandlerFactory
    )
    {
        $this->providerKey = $providerKey;
        $this->securityContext = $securityContext;
        $this->authenticationManager = $authenticationManager;
        $this->validator = $validator;
        $this->tokenTypeHandlerFactory = $tokenTypeHandlerFactory;
    }

    public function handle(GetResponseEvent $event)
    {
        $request = $event->getRequest();

        // Fetch access_token by token type handler.
        $tokenTypeHandler = $this->tokenTypeHandlerFactory->getTokenTypeHandler();
        $accessToken = $tokenTypeHandler->getAccessToken($request);

        // access_token must in valid format.
        $errors = $this->validator->validateValue($accessToken, array(
            new NotBlank(),
            new AccessToken(),
        ));
        if (count($errors) > 0) {
            throw new InvalidRequestException(array(
                'error_description' => 'The request includes an invalid parameter value.',
            ));
        }

        if (null !== $token = $this->securityContext->getToken()) {
            if ($token instanceof AccessTokenToken
                && $token->isAuthenticated()
                && $token->getAccessToken() === $accessToken
            ) {
                return;
            }
        }

        $token = new AccessTokenToken(
            $this->providerKey,
            $accessToken
        );
        $tokenAuthenticated = $this->authenticationManager->authenticate($token);
        $this->securityContext->setToken($tokenAuthenticated);
    }
}
